using Test
using TestSetExtensions
using IntensityMetrics
using IntensityMetrics.Unitful

@testset "perfect sinusoid" begin
    # ultrasound parameters
    f0 = 5e6u"Hz"                           # center frequency
    pressure_amplitude_1 = 1u"MPa"
    pressure_amplitude_2 = 2u"MPa"
    pressure_amplitude_3 = 3u"MPa"
    duty_cycle = 1.0
    pulse_duration = 10u"µs"
    total_duration = pulse_duration
    fs = 40e6u"Hz"                          # sampling frequency of received signal
    medium = Medium();
    excitation = Excitation(f0, pulse_duration, duty_cycle, total_duration, fs);

    # waveform parameters
    dt = 1/fs
    num_samples = ceil(pulse_duration/dt)
    time = collect((0:num_samples-1)*dt);
    data1 = sin.(2*pi*f0*time) * pressure_amplitude_1;
    data2 = cos.(2*pi*f0*time) * pressure_amplitude_2;
    data3 = cos.(2*pi*f0*time) * pressure_amplitude_3;
    data_all = repeat(hcat(data1, data2, data3), 1, 1, 2);


    @testset "intensity tests" begin
        intensity_1_expected = uconvert(u"W/m^2", pressure_amplitude_1^2 / (medium.density * medium.c))
        
        intensity_1_calculated = intensity(pressure_amplitude_1, medium)
        @test intensity_1_calculated ≈ intensity_1_expected atol=0.001u"W/m^2"

        intensity_1_calculated = intensity(uconvert(u"Pa", pressure_amplitude_1), medium)
        @test intensity_1_calculated ≈ intensity_1_expected atol=0.001u"W/m^2"
    end


    @testset "intensity_sppa tests" begin
        intensity_sppa_1_expected = uconvert(u"W/cm^2", pressure_amplitude_1^2 / (2 * medium.density * medium.c))
        intensity_sppa_2_expected = uconvert(u"W/cm^2", pressure_amplitude_2^2 / (2 * medium.density * medium.c))
        intensity_sppa_3_expected = uconvert(u"W/cm^2", pressure_amplitude_3^2 / (2 * medium.density * medium.c))
        intensity_sppa_expected = repeat(vcat(intensity_sppa_1_expected, intensity_sppa_2_expected, intensity_sppa_3_expected), 1, 2)

        # output is expected to be a vector
        intensity_sppa_1_calculated = intensity_sppa(data1, medium, excitation)
        @test intensity_sppa_1_calculated ≈ [intensity_sppa_1_expected] atol=0.001u"W/cm^2"

        intensity_sppa_calculated = intensity_sppa(data_all, medium, excitation)
        @test intensity_sppa_calculated ≈ intensity_sppa_expected atol=0.001u"W/cm^2"
    end


    @testset "mechanical_index tests" begin
        mechanical_index_1_expected = ustrip(uconvert(u"MPa", pressure_amplitude_1) / sqrt(uconvert(u"MHz", f0)))
        mechanical_index_2_expected = ustrip(uconvert(u"MPa", pressure_amplitude_2) / sqrt(uconvert(u"MHz", f0)))
        mechanical_index_3_expected = ustrip(uconvert(u"MPa", pressure_amplitude_3) / sqrt(uconvert(u"MHz", f0)))
        mechanical_index_expected = repeat(vcat(mechanical_index_1_expected, mechanical_index_2_expected, mechanical_index_3_expected), 1, 2)
        
        mechanical_index_1_calculated = mechanical_index(data1, excitation)
        @test mechanical_index_1_calculated ≈ [mechanical_index_1_expected] atol=0.001

        mechanical_index_1_calculated = mechanical_index(uconvert.(u"Pa", data1), excitation)
        @test mechanical_index_1_calculated ≈ [mechanical_index_1_expected] atol=0.001

        mechanical_index_calculated = mechanical_index(uconvert.(u"Pa", data_all), excitation)
        @test mechanical_index_calculated ≈ mechanical_index_expected atol=0.001
    end


    @testset "peak pos pressure tests" begin
        ppp_1_expected = pressure_amplitude_1
        ppp_2_expected = pressure_amplitude_2
        ppp_3_expected = pressure_amplitude_3
        ppp_expected = repeat(vcat(ppp_1_expected, ppp_2_expected, ppp_3_expected), 1, 2)

        @test pp_pressure(data1) ≈ [ppp_1_expected] atol=1u"Pa"
        @test pp_pressure(data_all) ≈ ppp_expected atol=1u"Pa"
    end
    

    @testset "peak neg pressure tests" begin
        pnp_1_expected = abs(-pressure_amplitude_1)
        pnp_2_expected = abs(-pressure_amplitude_2)
        pnp_3_expected = abs(-pressure_amplitude_3)
        pnp_expected = repeat(vcat(pnp_1_expected, pnp_2_expected, pnp_3_expected), 1, 2)
        
        @test pn_pressure(data1) ≈ [pnp_1_expected] atol=1u"Pa"
        @test pn_pressure(data_all) ≈ pnp_expected atol=1u"Pa"
    end    


    @testset "ptp pressure tests" begin
        ptp_pressure_1_expected = 2 * pressure_amplitude_1
        ptp_pressure_2_expected = 2 * pressure_amplitude_2
        ptp_pressure_3_expected = 2 * pressure_amplitude_3
        ptp_pressure_expected = repeat(vcat(ptp_pressure_1_expected, ptp_pressure_2_expected, ptp_pressure_3_expected), 1, 2)

        @test ptp_pressure(data1) ≈ [ptp_pressure_1_expected] atol=1u"Pa"
        @test ptp_pressure(data_all) ≈ ptp_pressure_expected atol=1u"Pa"
    end
end
